%option header-file="lex.yy.h"
%option noyywrap

%s COMMENT

%top{
  /* This goes at the top of the generated file */
  #include "parser.tab.h"

  /* The following is C code which is run whenever a token is matched.
  This feature is suitable for update yylloc variable as the parsing advances.
  yylloc is of type YYLTYPE which is a struct defined in this way:
  typedef struct YYLTYPE
  {
    int first_line;
    int first_column;
    int last_line;
    int last_column;
    char *filename;
  } YYLTYPE;
  */

  #define YY_USER_ACTION \
    yylloc.first_line = yylloc.last_line; \
    yylloc.first_column = yylloc.last_column; \
    for(int i = 0; yytext[i] != '\0'; i++) { \
        if(yytext[i] == '\n') { \
            yylloc.last_line++; \
            yylloc.last_column = 0; \
        } \
        else { \
            yylloc.last_column++; \
        } \
    }
}

atom_string [a-z][a-zA-Z0-9_]*
real [0-9]+"."[0-9]+
integer [0-9]+
number {real}|{integer}
sign "+"|"-"
exponent ("^"{number})
atom_number {sign}?{number}{exponent}?
variable [A-Z_][a-zA-Z0-9_]*

%%
<INITIAL>{
  {atom_string} {return ATOM;}
  {atom_number} {return ATOM;}
  {variable} {return VARIABLE;}
  "." {return DOT;}
  "?-" {return INT_SYMBOL;}
  "(" {return OP;}
  ")" {return CP;}
  ":-" {return R_SEPARATOR;}
  "\n"|"\r"|("\r\n") {;}
  "/*" {BEGIN(COMMENT);}
  , {return C;}
  . {;}
}
<COMMENT>{
  "*/" {BEGIN(INITIAL);}
  . {;}
}
